#!/bin/bash
# Copyright (c) 2019-2020 Oracle and/or its affiliates. All rights reserved.
# Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl.

set -o errexit
set -o nounset
set -o pipefail
set -o errtrace

# variables
REGISTRY=container-registry.oracle.com/olcne
K8S_VERSION=v1.17.9
FLANNEL_VERSION=v0.10.0-1
DASHBOARD_VERSION=v2.0.0
ETCD_VERSION=3.4.3
PAUSE_VERSION=3.1
COREDNS_VERSION=1.6.5
ISTIO_VERSION=1.4.10
ARCH=""
USELOCAL=0

function ops {
  local ops="${1:-}"
  local registry="${2:-${REGISTRY}}"

  for image in "${IMAGES[@]}"; do
  	podman "${ops}" "${registry}/${image}"
  done
}

function tag {
  local from="${1:-${REGISTRY}}"
  local to="${2:-}"

  for image in "${IMAGES[@]}"; do
  	podman tag "${from}/${image}" "${to}/${image}"
  done
}

function updatelist {
        IMAGENAMELIST="flannel:${FLANNEL_VERSION}${ARCH},\
etcd${ARCH}:${ETCD_VERSION},\
pause${ARCH}:${PAUSE_VERSION},\
kube-controller-manager${ARCH}:${K8S_VERSION},\
kube-scheduler${ARCH}:${K8S_VERSION},\
kube-apiserver${ARCH}:${K8S_VERSION},\
kube-proxy${ARCH}:${K8S_VERSION},\
kubernetes-dashboard:${DASHBOARD_VERSION},\
coredns:${COREDNS_VERSION},\
nginx:1.12.2,\
prometheus:v2.13.1,\
grafana:v6.7.4,\
sidecar_injector:${ISTIO_VERSION},\
proxyv2:${ISTIO_VERSION},\
proxy_init:${ISTIO_VERSION},\
pilot:${ISTIO_VERSION},\
mixer:${ISTIO_VERSION},\
istio_kubectl:${ISTIO_VERSION},\
galley:${ISTIO_VERSION},\
citadel:${ISTIO_VERSION}"
        IFS=',' read -d '' -ra IMAGES < <(printf '%s,\0' "$IMAGENAMELIST")
}

function check {
  local from="${1:-${REGISTRY}}"
  local to="${2:-}"
  local version="${3:-}"
  local image="pause"

  # start docker-engine
  echo "Checking if podman is installed ..."
  if ! podman --help > /dev/null; then
        echo "[ERROR] podman is not install ... please install podman"
        exit 1
  fi

  version=${version//v/}
  if  [ "${version}" = "1.13.10" ]; then
        K8S_VERSION=v${version}
        FLANNEL_VERSION=v0.10.0
        ETCD_VERSION=3.2.24
        PAUSE_VERSION=3.1
        DASHBOARD_VERSION=v1.10.1-1
        COREDNS_VERSION=1.2.6
        updatelist
  elif  [ "${version}" = "1.14.8" ]; then
        K8S_VERSION=v${version}
        FLANNEL_VERSION=v0.10.0-1
        ETCD_VERSION=3.3.10-1
        PAUSE_VERSION=3.1
        DASHBOARD_VERSION=v1.10.1-1
        COREDNS_VERSION=1.3.1
        updatelist
  elif  [ "${version}" = "1.14.9" ]; then
        K8S_VERSION=v${version}-2
        FLANNEL_VERSION=v0.10.0-1
        ETCD_VERSION=3.3.10-1
        PAUSE_VERSION=3.1
        DASHBOARD_VERSION=v1.10.1-1
        COREDNS_VERSION=1.3.1
        updatelist
  elif  [ "${version}" = "1.15.11" ]; then
        K8S_VERSION=v${version}
        FLANNEL_VERSION=v0.10.0-1
        ETCD_VERSION=3.3.10-1
        PAUSE_VERSION=3.1
        DASHBOARD_VERSION=v1.10.1-1
        COREDNS_VERSION=1.3.1
        updatelist
  elif  [ "${version}" = "1.16.8" ]; then
        K8S_VERSION=v${version}
        FLANNEL_VERSION=v0.10.0-1
        ETCD_VERSION=3.3.15
        PAUSE_VERSION=3.1
        DASHBOARD_VERSION=v1.10.1-1
        COREDNS_VERSION=1.6.2
        updatelist
  elif  [ "${version}" = "1.17.4" ]; then
        K8S_VERSION=v${version}
        FLANNEL_VERSION=v0.10.0-1
        ETCD_VERSION=3.4.3
        PAUSE_VERSION=3.1
        DASHBOARD_VERSION=v2.0.0
        COREDNS_VERSION=1.6.5
        updatelist
  elif  [ "${version}" = "1.17.6" ]; then
        K8S_VERSION=v${version}
        FLANNEL_VERSION=v0.10.0-1
        ETCD_VERSION=3.4.3
        PAUSE_VERSION=3.1
        DASHBOARD_VERSION=v2.0.0
        COREDNS_VERSION=1.6.5
        updatelist
  elif  [ "${version}" = "1.17.9" ]; then
        K8S_VERSION=v${version}
        FLANNEL_VERSION=v0.10.0-1
        ETCD_VERSION=3.4.3
        PAUSE_VERSION=3.1
        DASHBOARD_VERSION=v2.0.0
        COREDNS_VERSION=1.6.5
        updatelist
  elif  [ "v${version}" = "${K8S_VERSION}" ]; then
	:
  else
        echo "[ERROR] ${version} is not a valid version"
        exit 1
  fi

  echo "Checking for pull access from ${from} registry ..."
  # shellcheck disable=SC2155
  local result=$(echo ""|podman pull "${from}/${image}${ARCH}:${PAUSE_VERSION}" 2>&1|grep "Please login")
  if [ -n "${result}" ]; then
        echo "[ERROR] Please login with valid credential to the ${from}"
        echo "        # podman login ${from}"
        exit 1
  else
        if ! podman pull "${from}/${image}${ARCH}:${PAUSE_VERSION}"; then
                echo "[ERROR] cannot pull ${image}:${PAUSE_VERSION} from ${from} registry"
                exit 1
        fi
  fi

  podman tag "${from}/${image}${ARCH}:${PAUSE_VERSION}" "${to}/${image}${ARCH}:${PAUSE_VERSION}"

  echo "Checking for push access to ${to} registry ..."
  if ! podman push "${to}/${image}${ARCH}:${PAUSE_VERSION}"; then
	echo "[ERROR] cannot push ${image}${ARCH}:${PAUSE_VERSION} to ${to} registry"
	exit 1
  fi
}

function usage {
	echo "This script is to help pulling container images from default container-registry.oracle.com to a local repo" >&2
	echo "usage: " >&2
	echo "  $0 --to registry [--local --from registry --image-names <list of image:tag names> --version ${K8S_VERSION}]" >&2
	exit 1
}

# MAIN
ARG=($@)
PASS="0"
VERSION="${K8S_VERSION}"
FROM="${REGISTRY}"
# update the list with defaults
updatelist
for((i=0; i<$#; i++)); do
	if [ "${ARG[${i}]}" = "--to" ]; then
		PASS="1"
		i=$((i+1))
		TO="${ARG[${i}]:-}"
		if [ -z "${TO}" ]; then
			echo "[ERROR] Please provide a valid local registry location"
			exit 1
		fi
	elif [ "${ARG[${i}]}" = "--from" ]; then
		i=$((i+1))
		FROM="${ARG[${i}]:-${REGISTRY}}"
	elif [ "${ARG[${i}]}" = "--version" ]; then
		i=$((i+1))
		VERSION="${ARG[${i}]:-"${K8S_VERSION}"}"
	elif [ "${ARG[${i}]}" = "--local" ]; then
		i=$((i+1))
		USELOCAL=1
	elif [ "${ARG[${i}]}" = "--image-names" ]; then
		i=$((i+1))
		IFS=',' read -d '' -ra IMAGES < <(printf '%s,\0' "${ARG[${i}]:-}")
		if [ -z "${IMAGES[*]}" ]; then
			echo "[ERROR] Please provide a list of image:tag names"
			exit
		fi
	else
		usage
	fi
done

if [ "${PASS}" != "1" ]; then
	usage
fi

if [ "$USELOCAL" -eq 1 ]; then
        VERSION="${K8S_VERSION}"
        if ls /usr/local/share/kubeadm/*.tar &> /dev/null; then
                find /usr/local/share/kubeadm/*.tar -print0 | xargs -0 | xargs -n1 podman load -i
        fi
        if ls /usr/local/share/olcne/*.tar &> /dev/null; then
                find /usr/local/share/olcne/*.tar -print0 | xargs -0 | xargs -n1 podman load -i
        fi
else
	echo "Pulling images [${VERSION}] from: [${FROM}] to: [${TO}]"
	check "${FROM}" "${TO}" "${VERSION}"
	ops "pull" "${FROM}" # pull from registry
fi

tag "${FROM}" "${TO}"
ops "push" "${TO}" # push to registry
echo "[SUCCESS] All images pushed to [${TO}]"
